# -*- coding: utf-8 -*-
from django.core.management import BaseCommand
from ...utils import check_account_token_time_validity


class Command(BaseCommand):
    help = 'Checking token time validity and sand email if expired.'

    def handle(self, *args, **options):
        check_account_token_time_validity()



# -*- coding: utf-8 -*-
from django.core.management import BaseCommand
from ...utils import download_and_save_bank_transactions


class Command(BaseCommand):
    help = 'Download and save bank transactions.'

    def handle(self, *args, **options):
        download_and_save_bank_transactions()



# -*- coding: utf-8 -*-
import datetime
import unittest
from django.test import TestCase

mock_library_imported = False
try:
    from mock import patch
except ImportError:
    mock_library_imported = True

from ..models import Account, Transaction
from ..utils import check_account_token_time_validity,\
    download_and_save_bank_transactions


def _create_accounts():
    today = datetime.date.today()
    thirty_days_interval = datetime.timedelta(days=30)
    Account.objects.create(name='Tuhiik', account_number='3232',
                           bank_code='0021', token='f32#22',
                           token_expire=today + thirty_days_interval)

    Account.objects.create(name='My fio account', account_number='2323',
                           bank_code='2100', token='fsaretq2',
                           token_expire=today - thirty_days_interval)


def _destroy_accounts():
    Account.objects.all().delete()


class CheckTokenTimeValidityTest(TestCase):

    @classmethod
    def tearDownClass(cls):
        super(CheckTokenTimeValidityTest, cls).tearDownClass()
        _destroy_accounts()

    def test_send_email(self):
        _create_accounts()
        today = datetime.date.today()

        def fake_token_expired(account):
            if account.token_expire < today:
                self.assertGreater(today, account.token_expire)

        check_account_token_time_validity(email_sender=fake_token_expired)


class FioBankMock(object):

    def __init__(self, *args, **kwargs):
        super(FioBankMock, self).__init__()

    def last(self, from_id=None, from_date=None):
        return [{
                'comment': u'N\xe1kup:  \u010d\xe1stka 2769.00 CZK',
                'recipient_message': u'N\xe1kup:  \u010d\xe1stka 2769.00 CZK',
                'user_identifiaction': u'N\xe1kup:  \u010d\xe1stka 2769.00 CZK',
                'currency': 'CZK', 'amount': -2769.0,
                'instruction_id': '42424', 'executor': u'František Vocílka',
                'date': datetime.date.today(), 'type': u'Platba kartou',
                'transaction_id': '234242'}]


class DownloadAndSaveTransactionsTest(TestCase):

    @classmethod
    def tearDownClass(cls):
        super(DownloadAndSaveTransactionsTest, cls).tearDownClass()
        _destroy_accounts()

    @unittest.skipIf(mock_library_imported, 'Required mock library')
    @patch('fiobank.FioBank', FioBankMock)
    def test_run(self):
        _create_accounts()

        download_and_save_bank_transactions()

        transaction_list = Transaction.objects.all()
        self.assertEqual(1, len(transaction_list))



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'Transaction.account_number'
        db.alter_column(u'django_fiobank_transaction', 'account_number', self.gf('django.db.models.fields.CharField')(max_length=17))

    def backwards(self, orm):

        # Changing field 'Transaction.account_number'
        db.alter_column(u'django_fiobank_transaction', 'account_number', self.gf('django.db.models.fields.CharField')(max_length=16))

    models = {
        u'django_fiobank.account': {
            'Meta': {'unique_together': "(('account_number', 'bank_code'),)", 'object_name': 'Account'},
            'account_number': ('django.db.models.fields.CharField', [], {'max_length': '16'}),
            'bank_code': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'manager_email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'token': ('django.db.models.fields.CharField', [], {'max_length': '64'}),
            'token_expire': ('django.db.models.fields.DateField', [], {})
        },
        u'django_fiobank.transaction': {
            'Meta': {'unique_together': "(('instruction_id',),)", 'object_name': 'Transaction'},
            'account': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['django_fiobank.Account']"}),
            'account_name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'account_number': ('django.db.models.fields.CharField', [], {'max_length': '17'}),
            'amount': ('django.db.models.fields.FloatField', [], {}),
            'bank_code': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'bank_name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'bic': ('django.db.models.fields.CharField', [], {'max_length': '11', 'null': 'True', 'blank': 'True'}),
            'comment': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'constant_symbol': ('django.db.models.fields.CharField', [], {'max_length': '4', 'null': 'True', 'blank': 'True'}),
            'currency': ('django.db.models.fields.CharField', [], {'max_length': '3'}),
            'date': ('django.db.models.fields.DateField', [], {}),
            'executor': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instruction_id': ('django.db.models.fields.CharField', [], {'max_length': '15'}),
            'recipient_message': ('django.db.models.fields.CharField', [], {'max_length': '140', 'null': 'True', 'blank': 'True'}),
            'specific_symbol': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'specification': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'transaction_id': ('django.db.models.fields.BigIntegerField', [], {}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'user_identification': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'variable_symbol': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['django_fiobank']


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Account'
        db.create_table(u'django_fiobank_account', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('account_number', self.gf('django.db.models.fields.CharField')(max_length=16)),
            ('bank_code', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('token', self.gf('django.db.models.fields.CharField')(max_length=64)),
        ))
        db.send_create_signal(u'django_fiobank', ['Account'])

        # Adding unique constraint on 'Account', fields ['account_number', 'bank_code']
        db.create_unique(u'django_fiobank_account', ['account_number', 'bank_code'])

        # Adding model 'Transaction'
        db.create_table(u'django_fiobank_transaction', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('account', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['django_fiobank.Account'])),
            ('transaction_id', self.gf('django.db.models.fields.BigIntegerField')()),
            ('date', self.gf('django.db.models.fields.DateField')()),
            ('amount', self.gf('django.db.models.fields.FloatField')()),
            ('currency', self.gf('django.db.models.fields.CharField')(max_length=3)),
            ('account_number', self.gf('django.db.models.fields.CharField')(max_length=16)),
            ('account_name', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('bank_code', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('bic', self.gf('django.db.models.fields.CharField')(max_length=11, null=True, blank=True)),
            ('bank_name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('constant_symbol', self.gf('django.db.models.fields.CharField')(max_length=4, null=True, blank=True)),
            ('variable_symbol', self.gf('django.db.models.fields.CharField')(max_length=10, null=True, blank=True)),
            ('specific_symbol', self.gf('django.db.models.fields.CharField')(max_length=10, null=True, blank=True)),
            ('user_identification', self.gf('django.db.models.fields.CharField')(max_length=50, null=True, blank=True)),
            ('recipient_message', self.gf('django.db.models.fields.CharField')(max_length=140, null=True, blank=True)),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('executor', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('specification', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('comment', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('instruction_id', self.gf('django.db.models.fields.CharField')(max_length=15)),
        ))
        db.send_create_signal(u'django_fiobank', ['Transaction'])

        # Adding unique constraint on 'Transaction', fields ['instruction_id']
        db.create_unique(u'django_fiobank_transaction', ['instruction_id'])


    def backwards(self, orm):
        # Removing unique constraint on 'Transaction', fields ['instruction_id']
        db.delete_unique(u'django_fiobank_transaction', ['instruction_id'])

        # Removing unique constraint on 'Account', fields ['account_number', 'bank_code']
        db.delete_unique(u'django_fiobank_account', ['account_number', 'bank_code'])

        # Deleting model 'Account'
        db.delete_table(u'django_fiobank_account')

        # Deleting model 'Transaction'
        db.delete_table(u'django_fiobank_transaction')


    models = {
        u'django_fiobank.account': {
            'Meta': {'unique_together': "(('account_number', 'bank_code'),)", 'object_name': 'Account'},
            'account_number': ('django.db.models.fields.CharField', [], {'max_length': '16'}),
            'bank_code': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'token': ('django.db.models.fields.CharField', [], {'max_length': '64'})
        },
        u'django_fiobank.transaction': {
            'Meta': {'unique_together': "(('instruction_id',),)", 'object_name': 'Transaction'},
            'account': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['django_fiobank.Account']"}),
            'account_name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'account_number': ('django.db.models.fields.CharField', [], {'max_length': '16'}),
            'amount': ('django.db.models.fields.FloatField', [], {}),
            'bank_code': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'bank_name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'bic': ('django.db.models.fields.CharField', [], {'max_length': '11', 'null': 'True', 'blank': 'True'}),
            'comment': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'constant_symbol': ('django.db.models.fields.CharField', [], {'max_length': '4', 'null': 'True', 'blank': 'True'}),
            'currency': ('django.db.models.fields.CharField', [], {'max_length': '3'}),
            'date': ('django.db.models.fields.DateField', [], {}),
            'executor': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instruction_id': ('django.db.models.fields.CharField', [], {'max_length': '15'}),
            'recipient_message': ('django.db.models.fields.CharField', [], {'max_length': '140', 'null': 'True', 'blank': 'True'}),
            'specific_symbol': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'specification': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'transaction_id': ('django.db.models.fields.BigIntegerField', [], {}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'user_identification': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'variable_symbol': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['django_fiobank']


# -*- coding: utf-8 -*-
from django.core import mail
from django.core.mail import EmailMultiAlternatives
from django.test import TestCase

from ..models import Account
from ..emails import token_expired


class TokenExpiredTest(TestCase):

    def test_send_email(self):
        account = Account(manager_email='frantisek@vocilka.xxx')

        token_expired(account)

        self.assertTrue(hasattr(mail, 'outbox'))
        self.failUnlessEqual(len(mail.outbox), 1)
        self.assertIsInstance(mail.outbox[0], EmailMultiAlternatives)
        self.assertEqual(mail.outbox[0].to, [account.manager_email])



# -*- coding: utf-8 -*-
import datetime
import pygal
from pygal.style import Style
from django.db.models import Count, Sum
from django.utils.translation import ugettext as _
from django_fiobank.models import Transaction


def generate_income_issue_graph(account_id, year):
    date = datetime.datetime(year, 1, 1)
    income__transaction_list = Transaction.objects.extra(
        select={'year': 'EXTRACT(year FROM  CAST ("date" as DATE))',
                'month': 'EXTRACT(month from CAST ("date" as DATE))'}).values(
        'year', 'month').annotate(dcount=Count('date'), sum_amount=Sum(
        'amount')) \
        .order_by('year', 'month').filter(amount__gt=0,
                                          date__gte=date, account_id=account_id)

    issue_transaction_list = Transaction.objects.extra(
        select={'year': 'EXTRACT(year FROM  CAST ("date" as DATE))',
                'month': 'EXTRACT(month from CAST ("date" as DATE))'}).values(
        'year', 'month').annotate(dcount=Count('date'), sum_amount=Sum(
        'amount')) \
        .order_by('year', 'month').filter(amount__lt=0,
                                          date__gte=date, account_id=account_id)

    def month_to_str(month):
        return datetime.datetime(year, month, 1).strftime('%B')

    income_custom_list = {int(trans['month']): trans for trans in
                          income__transaction_list}
    issue_custom_list = {int(trans['month']): trans for trans in
                         issue_transaction_list}

    month_labels = []
    income_amount_list = []
    issue_amount_list = []
    for month_number in range(1, 12):
        month_labels.append(month_to_str(month_number))
        if month_number in income_custom_list:
            income_amount_list.append(
                income_custom_list[month_number]['sum_amount'])
        else:
            income_amount_list.append(None)

        if month_number in issue_custom_list:
            issue_amount_list.append(
                issue_custom_list[month_number]['sum_amount'] * -1)
        else:
            issue_amount_list.append(None)

    custom_style = Style(
        foreground='#FFFFFF',
        foreground_light='#fff',
        foreground_dark='#fff',
        opacity='.6',
        opacity_hover='.9',
        transition='400ms ease-in',
        colors=('#36E336', '#F72D05'))

    bar_chart = pygal.Bar(fill=True, interpolate='cubic',
                          style=custom_style)

    bar_chart.x_labels = month_labels
    bar_chart.add(_(u'Income'), income_amount_list)
    bar_chart.add(_(u'Issue'), issue_amount_list)
    return bar_chart.render()


# -*- coding: utf-8 -*-
import datetime
import logging
from django.db import IntegrityError
from django.db import transaction as db_transaction
from requests import HTTPError
import fiobank
from .models import Transaction, Account
from .emails import token_expired

logger = logging.getLogger(__name__)
if not logger.handlers:
    logger.addHandler(logging.StreamHandler())

token_expired_default_sender = token_expired


@db_transaction.commit_manually
def download_and_save_bank_transactions():
    try:
        today = datetime.date.today()
        account_list = Account.objects.all()
        for account in account_list:

            if account.token_expire < today:
                continue

            try:
                last_transaction = Transaction.objects.get_last_transaction(
                    account.id)

                client = fiobank.FioBank(token=account.token)

                if last_transaction:
                    lookup_params = {'from_id': last_transaction.transaction_id}
                    transaction_list = client.last(**lookup_params)
                else:
                    from_date = '{0}-01-01'.format(today.year)
                    transaction_list = client.last(from_date=from_date)

            except HTTPError, e:
                logger.error('Error in process account {0}'.format(account))
                logger.exception(e)
                break
            else:
                for data in transaction_list:
                    sid = db_transaction.savepoint()
                    try:
                        bank_trans = Transaction()
                        bank_trans.account = account
                        bank_trans.assign(data)
                        bank_trans.save()
                    except IntegrityError:
                        db_transaction.savepoint_rollback(sid)  # record exist
                    else:
                        db_transaction.savepoint_commit(sid)
    except Exception, e:
        db_transaction.rollback()
        raise e
    finally:
        db_transaction.commit()


def check_account_token_time_validity(
        email_sender=token_expired_default_sender):
    """ Check token validity and send emails if expired.

    :param email_sender: Function for send notification email. Default si
    token_expired.
    """
    minimal_validation_date = datetime.date.today() + datetime.timedelta(days=7)
    account_list = Account.objects.all()
    for account in account_list:
        if account.token_expire >= minimal_validation_date:
            email_sender(account)



# -*- coding: utf-8 -*-
import os
import sys
import shlex
import subprocess
import django_fiobank

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages  # NOQA

from setuptools.command.install_lib import install_lib as _install_lib
from distutils.command.build import build as _build
from distutils.cmd import Command

from django.core.management.commands.compilemessages import compile_messages

version = django_fiobank.__versionstr__


# release a version, publish to GitHub and PyPI
if sys.argv[-1] == 'publish':
    command = lambda cmd: subprocess.check_call(shlex.split(cmd))
    command('git tag v' + version)
    command('git push --tags origin master:master')
    command('python setup.py sdist upload')
    sys.exit()

base_path = os.path.dirname(__file__)


def read_file(filename):
    return open(os.path.join(base_path, filename)).read()


class CompileTranslations(Command):
    description = 'compile message catalogs to MO files via django ' \
                  'compilemessages'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        curdir = os.getcwd()
        os.chdir(os.path.realpath(os.path.dirname(django_fiobank.__file__)))
        try:
            compile_messages(stderr=sys.stderr)
        except TypeError:
            compile_messages(sys.stderr)
        os.chdir(curdir)


class Build(_build):
    sub_commands = [('compile_translations', None)] + _build.sub_commands


class InstallLib(_install_lib):
    def run(self):
        self.run_command('compile_translations')
        _install_lib.run(self)

setup(
    name='django_fiobank',
    version=version,
    description=('Django-app for managing and processing Fio Bank '
                 'transaction'),
    long_description=read_file('README.rst'),
    author='Martin Voldrich',
    author_email='rbas.cz@gmail.com',
    url='https://github.com/rbas/django-fiobank',
    packages=find_packages(),
    license=read_file('LICENSE'),
    install_requires=[
        'Django>=1.4', 'fiobank>=0.0.4,<0.1', 'south', 'pygal>=1.1.0,<2'
    ],
    include_package_data=True,
    package_data={'django_fiobank': ['locale/*/LC_MESSAGES/*']},
    cmdclass={'build': Build, 'install_lib': InstallLib,
              'compile_translations': CompileTranslations},
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet',
    )
)
